@@ -398,6 +398,13 @@ class IAMCredentialsClient(metaclass=IAMCredentialsClientMeta):
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.iam import credentials_v1
 
             def sample_generate_access_token():
@@ -407,7 +414,7 @@ class IAMCredentialsClient(metaclass=IAMCredentialsClientMeta):
                 # Initialize request argument(s)
                 request = credentials_v1.GenerateAccessTokenRequest(
                     name="name_value",
-                    scope=['scope_value_1', 'scope_value_2'],
+                    scope=['scope_value1', 'scope_value2'],
                 )
 
                 # Make the request
@@ -544,6 +551,13 @@ class IAMCredentialsClient(metaclass=IAMCredentialsClientMeta):
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.iam import credentials_v1
 
             def sample_generate_id_token():
@@ -683,6 +697,13 @@ class IAMCredentialsClient(metaclass=IAMCredentialsClientMeta):
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.iam import credentials_v1
 
             def sample_sign_blob():
@@ -809,6 +830,13 @@ class IAMCredentialsClient(metaclass=IAMCredentialsClientMeta):
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.iam import credentials_v1
 
             def sample_sign_jwt():
