@@ -13,43 +13,25 @@
 # limitations under the License.
 
 import collections
-import distutils
 import grpc
-import mock
+from unittest import mock
 import os
 import pytest
 
-import google.api_core.client_options as ClientOptions
+from google.api_core.client_options import ClientOptions  # type: ignore
 from google.auth import credentials
 from google.showcase import EchoClient
 from google.showcase import IdentityClient
 from google.showcase import MessagingClient
 
-if distutils.util.strtobool(os.environ.get("GAPIC_PYTHON_ASYNC", "true")):
+if os.environ.get("GAPIC_PYTHON_ASYNC", "true") == "true":
     from grpc.experimental import aio
     import asyncio
     from google.showcase import EchoAsyncClient
     from google.showcase import IdentityAsyncClient
 
-    @pytest.fixture
-    def async_echo(use_mtls, event_loop):
-        return construct_client(
-            EchoAsyncClient,
-            use_mtls,
-            transport="grpc_asyncio",
-            channel_creator=aio.insecure_channel
-        )
-
-    @pytest.fixture
-    def async_identity(use_mtls, event_loop):
-        return construct_client(
-            IdentityAsyncClient,
-            use_mtls,
-            transport="grpc_asyncio",
-            channel_creator=aio.insecure_channel
-        )
-
     _test_event_loop = asyncio.new_event_loop()
+    asyncio.set_event_loop(_test_event_loop)
 
     # NOTE(lidiz) We must override the default event_loop fixture from
     # pytest-asyncio. pytest fixture frees resources once there isn't any reference
@@ -58,9 +40,26 @@ if distutils.util.strtobool(os.environ.get("GAPIC_PYTHON_ASYNC", "true")):
 
     @pytest.fixture
     def event_loop():
-        asyncio.set_event_loop(_test_event_loop)
         return asyncio.get_event_loop()
 
+    @pytest.fixture
+    def async_echo(use_mtls, event_loop):
+        return construct_client(
+            EchoAsyncClient,
+            use_mtls,
+            transport_name="grpc_asyncio",
+            channel_creator=aio.insecure_channel
+        )
+
+    @pytest.fixture
+    def async_identity(use_mtls, event_loop):
+        return construct_client(
+            IdentityAsyncClient,
+            use_mtls,
+            transport_name="grpc_asyncio",
+            channel_creator=aio.insecure_channel
+        )
+
 
 dir = os.path.dirname(__file__)
 with open(os.path.join(dir, "../cert/mtls.crt"), "rb") as fh:
@@ -77,7 +76,7 @@ def callback():
     return cert, key
 
 
-client_options = ClientOptions.ClientOptions()
+client_options = ClientOptions()
 client_options.client_cert_source = callback
 
 
@@ -87,10 +86,12 @@ def pytest_addoption(parser):
     )
 
 
-def construct_client(client_class,
-                     use_mtls,
-                     transport="grpc",
-                     channel_creator=grpc.insecure_channel):
+def construct_client(
+    client_class,
+    use_mtls,
+    transport_name="grpc",
+    channel_creator=grpc.insecure_channel,
+):
     if use_mtls:
         with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
             with mock.patch("grpc.ssl_channel_credentials", autospec=True) as mock_ssl_cred:
@@ -104,9 +105,22 @@ def construct_client(client_class,
                 )
                 return client
     else:
-        transport = client_class.get_transport_class(transport)(
-            channel=channel_creator("localhost:7469")
-        )
+        transport_cls = client_class.get_transport_class(transport_name)
+        if transport_name in ["grpc", "grpc_asyncio"]:
+            transport = transport_cls(
+                credentials=credentials.AnonymousCredentials(),
+                channel=channel_creator("localhost:7469"),
+            )
+        elif transport_name == "rest":
+            # The custom host explicitly bypasses https.
+            transport = transport_cls(
+                credentials=credentials.AnonymousCredentials(),
+                host="localhost:7469",
+                url_scheme="http",
+            )
+        else:
+            raise RuntimeError(f"Unexpected transport type: {transport_name}")
+
         return client_class(transport=transport)
 
 
@@ -115,27 +129,19 @@ def use_mtls(request):
     return request.config.getoption("--mtls")
 
 
-@pytest.fixture
-def echo(use_mtls):
-    return construct_client(EchoClient, use_mtls)
+@pytest.fixture(params=["grpc", "rest"])
+def echo(use_mtls, request):
+    return construct_client(EchoClient, use_mtls, transport_name=request.param)
 
 
-@pytest.fixture
-def identity():
-    transport = IdentityClient.get_transport_class('grpc')(
-        channel=grpc.insecure_channel('localhost:7469'),
-    )
-    return IdentityClient(transport=transport)
+@pytest.fixture(params=["grpc", "rest"])
+def identity(use_mtls, request):
+    return construct_client(IdentityClient, use_mtls, transport_name=request.param)
 
 
-@pytest.fixture
-def identity(use_mtls):
-    return construct_client(IdentityClient, use_mtls)
-
-
-@pytest.fixture
-def messaging(use_mtls):
-    return construct_client(MessagingClient, use_mtls)
+@pytest.fixture(params=["grpc", "rest"])
+def messaging(use_mtls, request):
+    return construct_client(MessagingClient, use_mtls, transport_name=request.param)
 
 
 class MetadataClientInterceptor(
