@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import json
 from textwrap import dedent
 from typing import Mapping
 from unittest import mock
@@ -19,17 +20,30 @@ from unittest import mock
 import jinja2
 import pytest
 
+from google.api import service_pb2
 from google.protobuf import descriptor_pb2
 from google.protobuf.compiler.plugin_pb2 import CodeGeneratorResponse
 
 from gapic.generator import generator
-from gapic.samplegen_utils import types, yaml
+from gapic.samplegen_utils import snippet_metadata_pb2, types, yaml
+from ..common_types import (DummyApiSchema, DummyField, DummyIdent, DummyNaming, DummyMessage, DummyMessageTypePB,
+                          DummyService, DummyMethod, message_factory, enum_factory)
+
 from gapic.schema import api
 from gapic.schema import naming
 from gapic.schema import wrappers
 from gapic.utils import Options
 
 
+def mock_generate_sample(*args, **kwargs):
+    dummy_snippet_metadata = snippet_metadata_pb2.Snippet()
+    dummy_snippet_metadata.client_method.method.service.short_name = args[0]["service"].split(
+        ".")[-1]
+    dummy_snippet_metadata.client_method.method.short_name = args[0]['rpc']
+
+    return "", dummy_snippet_metadata
+
+
 def test_custom_template_directory():
     # Create a generator.
     opts = Options.build("python-gapic-templates=/templates/")
@@ -50,8 +64,8 @@ def test_get_response():
             lt.assert_called_once()
             gt.assert_has_calls(
                 [
-                    mock.call("foo/bar/baz.py.j2"),
                     mock.call("molluscs/squid/sample.py.j2"),
+                    mock.call("foo/bar/baz.py.j2"),
                 ]
             )
             assert len(cgr.file) == 1
@@ -70,8 +84,8 @@ def test_get_response_ignores_empty_files():
             lt.assert_called_once()
             gt.assert_has_calls(
                 [
-                    mock.call("foo/bar/baz.py.j2"),
                     mock.call("molluscs/squid/sample.py.j2"),
+                    mock.call("foo/bar/baz.py.j2"),
                 ]
             )
             assert len(cgr.file) == 0
@@ -92,8 +106,8 @@ def test_get_response_ignores_private_files():
             lt.assert_called_once()
             gt.assert_has_calls(
                 [
-                    mock.call("foo/bar/baz.py.j2"),
                     mock.call("molluscs/squid/sample.py.j2"),
+                    mock.call("foo/bar/baz.py.j2"),
                 ]
             )
             assert len(cgr.file) == 1
@@ -106,7 +120,6 @@ def test_get_response_fails_invalid_file_paths():
     with mock.patch.object(jinja2.FileSystemLoader, "list_templates") as lt:
         lt.return_value = [
             "foo/bar/%service/%proto/baz.py.j2",
-            "molluscs/squid/sample.py.j2",
         ]
         with pytest.raises(ValueError) as ex:
             g.get_response(api_schema=make_api(),
@@ -116,6 +129,77 @@ def test_get_response_fails_invalid_file_paths():
         assert "%proto" in ex_str and "%service" in ex_str
 
 
+def test_get_response_ignore_gapic_metadata():
+    g = make_generator()
+    with mock.patch.object(jinja2.FileSystemLoader, "list_templates") as lt:
+        lt.return_value = ["gapic/gapic_metadata.json.j2"]
+        with mock.patch.object(jinja2.Environment, "get_template") as gt:
+            gt.return_value = jinja2.Template(
+                "This is not something we want to see")
+            res = g.get_response(
+                api_schema=make_api(),
+                opts=Options.build(""),
+            )
+
+            # We don't expect any files because opts.metadata is not set.
+            assert res.file == CodeGeneratorResponse().file
+
+
+def test_get_response_ignores_unwanted_transports_and_clients():
+    g = make_generator()
+    with mock.patch.object(jinja2.FileSystemLoader, "list_templates") as lt:
+        lt.return_value = [
+            "foo/%service/transports/river.py.j2",
+            "foo/%service/transports/car.py.j2",
+            "foo/%service/transports/grpc.py.j2",
+            "foo/%service/transports/__init__.py.j2",
+            "foo/%service/transports/base.py.j2",
+            "foo/%service/async_client.py.j2",
+            "foo/%service/client.py.j2",
+            "mollusks/squid/sample.py.j2",
+        ]
+
+        with mock.patch.object(jinja2.Environment, "get_template") as gt:
+            gt.return_value = jinja2.Template("Service: {{ service.name }}")
+            api_schema = make_api(
+                make_proto(
+                    descriptor_pb2.FileDescriptorProto(
+                        service=[
+                            descriptor_pb2.ServiceDescriptorProto(
+                                name="SomeService"),
+                        ]
+                    ),
+                )
+            )
+
+            cgr = g.get_response(
+                api_schema=api_schema,
+                opts=Options.build("transport=river+car")
+            )
+            assert len(cgr.file) == 5
+            assert {i.name for i in cgr.file} == {
+                "foo/some_service/transports/river.py",
+                "foo/some_service/transports/car.py",
+                "foo/some_service/transports/__init__.py",
+                "foo/some_service/transports/base.py",
+                # Only generate async client with grpc transport
+                "foo/some_service/client.py",
+            }
+
+            cgr = g.get_response(
+                api_schema=api_schema,
+                opts=Options.build("transport=grpc")
+            )
+            assert len(cgr.file) == 5
+            assert {i.name for i in cgr.file} == {
+                "foo/some_service/transports/grpc.py",
+                "foo/some_service/transports/__init__.py",
+                "foo/some_service/transports/base.py",
+                "foo/some_service/client.py",
+                "foo/some_service/async_client.py",
+            }
+
+
 def test_get_response_enumerates_services():
     g = make_generator()
     with mock.patch.object(jinja2.FileSystemLoader, "list_templates") as lt:
@@ -171,7 +255,10 @@ def test_get_response_enumerates_proto():
 
 
 def test_get_response_divides_subpackages():
-    g = make_generator()
+    # NOTE: autogen-snippets is intentionally disabled for this test
+    # The API schema below is incomplete and will result in errors when the
+    # snippetgen logic tries to parse it.
+    g = make_generator("autogen-snippets=false")
     api_schema = api.API.build(
         [
             descriptor_pb2.FileDescriptorProto(
@@ -206,7 +293,7 @@ def test_get_response_divides_subpackages():
             """.strip()
             )
             cgr = g.get_response(api_schema=api_schema,
-                                 opts=Options.build(""))
+                                 opts=Options.build("autogen-snippets=false"))
             assert len(cgr.file) == 6
             assert {i.name for i in cgr.file} == {
                 "foo/types/top.py",
@@ -331,13 +418,8 @@ def test_parse_sample_paths(fs):
         Options.build("samples=sampledir/,")
 
 
-@mock.patch(
-    "gapic.samplegen.samplegen.generate_sample", return_value="",
-)
 @mock.patch("time.gmtime",)
-def test_samplegen_config_to_output_files(
-    mock_gmtime, mock_generate_sample, fs,
-):
+def test_samplegen_config_to_output_files(mock_gmtime, fs):
     # These time values are nothing special,
     # they just need to be deterministic.
     returner = mock.MagicMock()
@@ -359,9 +441,11 @@ def test_samplegen_config_to_output_files(
             samples:
             - id: squid_sample
               region_tag: humboldt_tag
-              rpc: get_squid_streaming
+              service: Mollusc.v1.Mollusc
+              rpc: GetSquidStreaming
             - region_tag: clam_sample
-              rpc: get_clam
+              service: Mollusc.v1.Mollusc
+              rpc: GetClam
             """
         ),
     )
@@ -370,54 +454,117 @@ def test_samplegen_config_to_output_files(
     # Need to have the sample template visible to the generator.
     g._env.loader = jinja2.DictLoader({"sample.py.j2": ""})
 
-    api_schema = make_api(naming=naming.NewNaming(
-        name="Mollusc", version="v6"))
-    actual_response = g.get_response(
-        api_schema, opts=Options.build(""))
-    expected_response = CodeGeneratorResponse(
-        file=[
-            CodeGeneratorResponse.File(
-                name="samples/squid_sample.py", content="\n",),
-            CodeGeneratorResponse.File(
-                name="samples/clam_sample.py", content="\n",),
-            CodeGeneratorResponse.File(
-                name="samples/mollusc.v6.python.21120601.131313.manifest.yaml",
-                content=dedent(
-                    """\
-                ---
-                type: manifest/samples
-                schema_version: 3
-                python: &python
-                  environment: python
-                  bin: python3
-                  base_path: samples
-                  invocation: '{bin} {path} @args'
-                samples:
-                - <<: *python
-                  sample: squid_sample
-                  path: '{base_path}/squid_sample.py'
-                  region_tag: humboldt_tag
-                - <<: *python
-                  sample: clam_sample
-                  path: '{base_path}/clam_sample.py'
-                  region_tag: clam_sample
-                """
-                ),
-            ),
-        ]
-    )
-    expected_response.supported_features |= (
-        CodeGeneratorResponse.Feature.FEATURE_PROTO3_OPTIONAL
+    api_schema = DummyApiSchema(
+        services={"Mollusc": DummyService(
+            name="Mollusc",
+            methods={
+                # For this test the generator only cares about the dictionary keys
+                "GetSquidStreaming": DummyMethod(),
+                "GetClam": DummyMethod(),
+            },
+            )},
+        naming=DummyNaming(name="mollusc", version="v1", warehouse_package_name="mollusc-cephalopod-teuthida-",
+                           versioned_module_name="teuthida_v1", module_namespace="mollusc.cephalopod", proto_package="google.mollusca"),
     )
 
-    assert actual_response == expected_response
+    with mock.patch("gapic.samplegen.samplegen.generate_sample", side_effect=mock_generate_sample):
+        actual_response = g.get_response(
+            api_schema, opts=Options.build("autogen-snippets=False"))
+
+    expected_snippet_index_json = {
+        "clientLibrary": {
+            "apis": [{
+                "id": "google.mollusca",
+                "version": "v1"
+            }],
+            "language": "PYTHON",
+            "name": "mollusc-cephalopod-teuthida-"
+            },
+        "snippets": [
+            {
+                "clientMethod": {
+                    "method": {
+                        "service": {
+                            "shortName": "Mollusc"
+                            },
+                        "shortName": "GetSquidStreaming"
+                        }
+                    },
+                "file": "squid_sample.py",
+                "segments": [
+                    {"type": "FULL"},
+                    {"type": "SHORT"},
+                    {"type": "CLIENT_INITIALIZATION"},
+                    {"type": "REQUEST_INITIALIZATION"},
+                    {"type": "REQUEST_EXECUTION"},
+                    {"type": "RESPONSE_HANDLING"}
+                ],
+                "title": "squid_sample.py"
+                },
+            {
+                "clientMethod": {
+                    "method": {
+                        "service": {
+                            "shortName": "Mollusc"
+                            },
+                        "shortName": "GetClam"
+                        }
+                    },
+                "file": "clam_sample.py",
+                "segments": [
+                    {"type": "FULL"},
+                    {"type": "SHORT"},
+                    {"type": "CLIENT_INITIALIZATION"},
+                    {"type": "REQUEST_INITIALIZATION"},
+                    {"type": "REQUEST_EXECUTION"},
+                    {"type": "RESPONSE_HANDLING"}
+                ],
+                "title": "clam_sample.py"
+                }
+            ]
+        }
+
+    assert actual_response.supported_features == CodeGeneratorResponse.Feature.FEATURE_PROTO3_OPTIONAL
+
+    assert len(actual_response.file) == 3
+    assert actual_response.file[0] == CodeGeneratorResponse.File(
+        name="samples/generated_samples/squid_sample.py", content="\n",)
+    assert actual_response.file[1] == CodeGeneratorResponse.File(
+        name="samples/generated_samples/clam_sample.py", content="\n",)
+
+    assert actual_response.file[2].name == "samples/generated_samples/snippet_metadata_mollusc_v1.json"
+
+    assert json.loads(
+        actual_response.file[2].content) == expected_snippet_index_json
 
 
 @mock.patch(
-    "gapic.samplegen.samplegen.generate_sample", return_value="",
+    "gapic.samplegen.samplegen.generate_sample_specs", return_value=[]
 )
+@mock.patch(
+    "gapic.samplegen.samplegen.generate_sample", return_value=("", snippet_metadata_pb2.Snippet()),
+)
+def test_generate_autogen_samples(mock_generate_sample, mock_generate_specs):
+    opts = Options.build("autogen-snippets")
+    g = generator.Generator(opts)
+    # Need to have the sample template visible to the generator.
+    g._env.loader = jinja2.DictLoader({"sample.py.j2": ""})
+
+    api_schema = make_api(naming=naming.NewNaming(
+        name="Mollusc", version="v6"))
+
+    actual_response = g.get_response(api_schema, opts=opts)
+
+    # Just check that generate_sample_specs was called
+    # Correctness of the spec is tested in samplegen unit tests
+    mock_generate_specs.assert_called_once_with(
+        api_schema,
+        opts=opts
+    )
+
+
 @mock.patch("time.gmtime",)
-def test_samplegen_id_disambiguation(mock_gmtime, mock_generate_sample, fs):
+def test_samplegen_id_disambiguation(mock_gmtime, fs):
     # These time values are nothing special,
     # they just need to be deterministic.
     returner = mock.MagicMock()
@@ -443,12 +590,15 @@ def test_samplegen_id_disambiguation(mock_gmtime, mock_generate_sample, fs):
             samples:
             - id: squid_sample
               region_tag: humboldt_tag
-              rpc: get_squid_streaming
+              rpc: GetSquidStreaming
+              service: Mollusc.v1.Mollusc
             # Note that this region tag collides with the id of the previous sample.
             - region_tag: squid_sample
-              rpc: get_squid_streaming
+              rpc: GetSquidStreaming
+              service: Mollusc.v1.Mollusc
             # No id or region tag.
-            - rpc: get_squid_streaming
+            - rpc: GetSquidStreaming
+              service: Mollusc.v1.Mollusc
             """
         ),
     )
@@ -456,54 +606,112 @@ def test_samplegen_id_disambiguation(mock_gmtime, mock_generate_sample, fs):
     # Need to have the sample template visible to the generator.
     g._env.loader = jinja2.DictLoader({"sample.py.j2": ""})
 
-    api_schema = make_api(naming=naming.NewNaming(
-        name="Mollusc", version="v6"))
-    actual_response = g.get_response(api_schema,
-                                     opts=Options.build(""))
-    expected_response = CodeGeneratorResponse(
-        file=[
-            CodeGeneratorResponse.File(
-                name="samples/squid_sample_91a465c6.py", content="\n",
-            ),
-            CodeGeneratorResponse.File(
-                name="samples/squid_sample_55051b38.py", content="\n",
-            ),
-            CodeGeneratorResponse.File(name="samples/157884ee.py",
-                                       content="\n",),
-            CodeGeneratorResponse.File(
-                name="samples/mollusc.v6.python.21120601.131313.manifest.yaml",
-                content=dedent(
-                    """\
-                ---
-                type: manifest/samples
-                schema_version: 3
-                python: &python
-                  environment: python
-                  bin: python3
-                  base_path: samples
-                  invocation: '{bin} {path} @args'
-                samples:
-                - <<: *python
-                  sample: squid_sample_91a465c6
-                  path: '{base_path}/squid_sample_91a465c6.py'
-                  region_tag: humboldt_tag
-                - <<: *python
-                  sample: squid_sample_55051b38
-                  path: '{base_path}/squid_sample_55051b38.py'
-                  region_tag: squid_sample
-                - <<: *python
-                  sample: 157884ee
-                  path: '{base_path}/157884ee.py'
-                """
-                ),
-            ),
-        ]
-    )
-    expected_response.supported_features |= (
-        CodeGeneratorResponse.Feature.FEATURE_PROTO3_OPTIONAL
+    api_schema = DummyApiSchema(
+        services={"Mollusc": DummyService(
+            name="Mollusc",
+            methods={
+                # The generator only cares about the dictionary keys
+                "GetSquidStreaming": DummyMethod(),
+                "GetClam": DummyMethod(),
+            },
+            )},
+        naming=DummyNaming(name="mollusc", version="v1", warehouse_package_name="mollusc-cephalopod-teuthida-",
+                           versioned_module_name="teuthida_v1", module_namespace="mollusc.cephalopod", proto_package="google.mollusca"),
     )
+    with mock.patch("gapic.samplegen.samplegen.generate_sample", side_effect=mock_generate_sample):
+        actual_response = g.get_response(api_schema,
+                                     opts=Options.build("autogen-snippets=False"))
 
-    assert actual_response == expected_response
+    expected_snippet_metadata_json = {
+        "clientLibrary": {
+            "apis": [
+                {
+                    "id": "google.mollusca",
+                    "version": "v1"
+                    }
+            ],
+            "language": "PYTHON",
+            "name": "mollusc-cephalopod-teuthida-"
+        },
+        "snippets": [
+            {
+                "clientMethod": {
+                    "method": {
+                        "service": {
+                            "shortName": "Mollusc"
+                            },
+                        "shortName": "GetSquidStreaming"
+                    }
+                },
+                "file": "squid_sample_1cfd0b3d.py",
+                "segments": [
+                    {"type": "FULL"},
+                    {"type": "SHORT"},
+                    {"type": "CLIENT_INITIALIZATION"},
+                    {"type": "REQUEST_INITIALIZATION"},
+                    {"type": "REQUEST_EXECUTION"},
+                    {"type": "RESPONSE_HANDLING"}
+                ],
+                "title": "squid_sample_1cfd0b3d.py"
+                },
+            {
+                "clientMethod": {
+                    "method": {
+                        "service": {
+                            "shortName": "Mollusc"
+                            },
+                        "shortName": "GetSquidStreaming"
+                    }
+                },
+                "file": "squid_sample_cf4d4fa4.py",
+                "segments": [
+                    {"type": "FULL"},
+                    {"type": "SHORT"},
+                    {"type": "CLIENT_INITIALIZATION"},
+                    {"type": "REQUEST_INITIALIZATION"},
+                    {"type": "REQUEST_EXECUTION"},
+                    {"type": "RESPONSE_HANDLING"}
+                ],
+                "title": "squid_sample_cf4d4fa4.py"
+                },
+            {
+                "clientMethod": {
+                    "method": {
+                        "service": {
+                            "shortName": "Mollusc"
+                            },
+                        "shortName": "GetSquidStreaming"
+                    }
+                },
+                "file": "7384949e.py",
+                "segments": [
+                    {"type": "FULL"},
+                    {"type": "SHORT"},
+                    {"type": "CLIENT_INITIALIZATION"},
+                    {"type": "REQUEST_INITIALIZATION"},
+                    {"type": "REQUEST_EXECUTION"},
+                    {"type": "RESPONSE_HANDLING"}
+                ],
+                "title": "7384949e.py"
+                }
+            ]
+    }
+
+    assert actual_response.supported_features == CodeGeneratorResponse.Feature.FEATURE_PROTO3_OPTIONAL
+    assert len(actual_response.file) == 4
+    assert actual_response.file[0] == CodeGeneratorResponse.File(
+        name="samples/generated_samples/squid_sample_1cfd0b3d.py", content="\n",
+    )
+    assert actual_response.file[1] == CodeGeneratorResponse.File(
+        name="samples/generated_samples/squid_sample_cf4d4fa4.py", content="\n",
+    )
+    assert actual_response.file[2] == CodeGeneratorResponse.File(
+        name="samples/generated_samples/7384949e.py", content="\n",
+    )
+    print(actual_response.file[3].content)
+    assert actual_response.file[3].name == "samples/generated_samples/snippet_metadata_mollusc_v1.json"
+    assert json.loads(
+        actual_response.file[3].content) == expected_snippet_metadata_json
 
 
 def test_generator_duplicate_samples(fs):
@@ -536,113 +744,6 @@ def test_generator_duplicate_samples(fs):
                                opts=Options.build(""))
 
 
-@mock.patch("gapic.samplegen.samplegen.generate_sample", return_value="")
-@mock.patch("time.gmtime",)
-def test_dont_generate_in_code_samples(mock_gmtime, mock_generate_sample, fs):
-    # These time values are nothing special,
-    # they just need to be deterministic.
-    returner = mock.MagicMock()
-    returner.tm_year = 2112
-    returner.tm_mon = 6
-    returner.tm_mday = 1
-    returner.tm_hour = 13
-    returner.tm_min = 13
-    returner.tm_sec = 13
-    mock_gmtime.return_value = returner
-
-    config_fpath = "samples.yaml"
-    fs.create_file(
-        config_fpath,
-        contents=dedent(
-            """
-            type: com.google.api.codegen.samplegen.v1p2.SampleConfigProto
-            schema_version: 1.2.0
-            samples:
-            - id: squid_sample
-              rpc: IdentifyMollusc
-              service: Mollusc.v1.Mollusc
-              sample_type:
-              - standalone
-              - incode/SQUID
-            - id: clam_sample
-              rpc: IdentifyMollusc
-              service: Mollusc.v1.Mollusc
-              sample_type:
-              - incode/CLAM
-            - id: whelk_sample
-              rpc: IdentifyMollusc
-              service: Mollusc.v1.Mollusc
-              sample_type:
-              - standalone
-            - id: octopus_sample
-              rpc: IdentifyMollusc
-              service: Mollusc.v1.Mollusc
-            """
-        ),
-    )
-
-    generator = make_generator(f"samples={config_fpath}")
-    generator._env.loader = jinja2.DictLoader({"sample.py.j2": ""})
-    api_schema = make_api(
-        make_proto(
-            descriptor_pb2.FileDescriptorProto(
-                name="mollusc.proto",
-                package="Mollusc.v1",
-                service=[descriptor_pb2.ServiceDescriptorProto(
-                    name="Mollusc")],
-            ),
-        ),
-        naming=naming.NewNaming(name="Mollusc", version="v6"),
-    )
-
-    # Note that we do NOT expect a clam sample.
-    # There are four tests going on:
-    # 1) Just an explicit standalone sample type.
-    # 2) Multiple sample types, one of which is standalone.
-    # 3) Explicit sample types but NO standalone sample type.
-    # 4) Implicit standalone sample type.
-    expected = CodeGeneratorResponse(
-        file=[
-            CodeGeneratorResponse.File(
-                name="samples/squid_sample.py", content="\n",),
-            CodeGeneratorResponse.File(
-                name="samples/whelk_sample.py", content="\n",),
-            CodeGeneratorResponse.File(
-                name="samples/octopus_sample.py", content="\n",),
-            CodeGeneratorResponse.File(
-                name="samples/mollusc.v6.python.21120601.131313.manifest.yaml",
-                content=dedent(
-                    """                ---
-                type: manifest/samples
-                schema_version: 3
-                python: &python
-                  environment: python
-                  bin: python3
-                  base_path: samples
-                  invocation: \'{bin} {path} @args\'
-                samples:
-                - <<: *python
-                  sample: squid_sample
-                  path: \'{base_path}/squid_sample.py\'
-                - <<: *python
-                  sample: whelk_sample
-                  path: \'{base_path}/whelk_sample.py\'
-                - <<: *python
-                  sample: octopus_sample
-                  path: \'{base_path}/octopus_sample.py\'
-                """
-                ),
-            ),
-        ]
-    )
-    expected.supported_features |= CodeGeneratorResponse.Feature.FEATURE_PROTO3_OPTIONAL
-
-    actual = generator.get_response(
-        api_schema=api_schema, opts=Options.build("")
-    )
-    assert actual == expected
-
-
 def make_generator(opts_str: str = "") -> generator.Generator:
     return generator.Generator(Options.build(opts_str))
 
@@ -662,9 +763,17 @@ def make_proto(
     ).proto
 
 
-def make_api(*protos, naming: naming.Naming = None, **kwargs) -> api.API:
+def make_api(
+    *protos,
+    naming: naming.Naming = None,
+    service_yaml_config: service_pb2.Service = None,
+    **kwargs
+) -> api.API:
     return api.API(
-        naming=naming or make_naming(), all_protos={i.name: i for i in protos}, **kwargs
+        naming=naming or make_naming(),
+        service_yaml_config=service_yaml_config or service_pb2.Service(),
+        all_protos={i.name: i for i in protos},
+        **kwargs
     )
 
 
